-- Player class
-- Used to control the player character
-- With code from Sean Laurvick and YellowAfterlife

local RUN_SPEED = 500 -- Run acceleration
local MAX_SPEED = 160 -- Maximum running speed
local BRAKE_SPEED = 250
local JUMP_POWER = 450 -- initial yspeed when jumping

PC_STANDING, PC_MOVING, PC_DEAD = 0,1,2
local lg = love.graphics

Player = {}

-- Constructor
function Player:new()
	-- define our parameters here
	local object = {
		-- dimensions:
		x = 0,
		y = 0,
		width = 20,
		height = 20,

		-- original position:
		xstart = 0,
		ystart = 0,
		
		-- movement:
		xspeed = 0,
		yspeed = 0,
		dir = 1,
		lastDir = 1,
		state = PC_STANDING,
		onGround = false,
		
		-- hit detection:
		corners = {-6, 5, -22, -0.5},
		
		-- animation:
		animStand		= newAnimation(img.player, 20, 20, 0.12, 1),
		animRun		= newAnimation(img.player, 20, 20, 0.12, 1),
		anim = nil
	}
	setmetatable(object, { __index = Player })
	return object
end

function Player:update(gravity, dt)
	-- MOVING STATE, STANDING STATE
	if self.state == PC_MOVING or self.state == PC_STANDING then
		self:updateMovement(gravity, dt)
	-- DEAD STATE
	elseif self.state == PS_DEAD then
		-- Nothing yet
	end

end

--- Called each update if current state is PC_MOVING or PC_STANDING
-- @param gravity Standard global gravity
-- @param dt Time passed since last update
function Player:updateMovement(gravity, dt)
	local changedDir = false -- true if player changed horizontal direction

	-- Check if both directions are held for handling conflicts
	local both = keystate.right and keystate.left
	-- Walk left
	if (both == false and keystate.right) or (both == true and self.lastDir == 1) then
		self.xspeed = self.xspeed + RUN_SPEED*dt

		if self.dir == -1 then
			self.dir = 1
			changedDir = true
		end
		self:setState(PC_MOVING)
	-- Walk right
	elseif (both == false and keystate.left) or (both == true and self.lastDir == -1) then
		self.xspeed = self.xspeed - RUN_SPEED*dt

		if self.dir == 1 then
			self.dir = -1
			changedDir = true
		end
		self:setState(PC_MOVING)
	end

	-- Cap speeds
	if self.xspeed < 0 then
		self.xspeed = self.xspeed + math.max(dt*BRAKE_SPEED, self.xspeed)
	elseif self.xspeed > 0 then
		self.xspeed = self.xspeed - math.min(dt*BRAKE_SPEED, self.xspeed)
	elseif self.speed == 0 then
		self:setState(PC_STANDING)
	end

	-- Move in x axis
	if self:collideX(dt) == false then
		self.x = self.x + (self.xspeed*dt)
	else
		self.xspeed = -0.25*self.xspeed
	end
	-- Update gravity
	self.yspeed = self.yspeed + (gravity*dt)
	-- Move in y axis
	if self:collideY(dt) == false then
		self.y = self.y + (self.yspeed*dt)
	else
		self.yspeed = 0
		self:setState(PC_STANDING)
	end

	-- Set animation speeds
	self.anim:setSpeed(math.abs(self.xspeed)/MAX_SPEED)
end

-- Check all around the player, since they often have momentum
function Player:collideX(dt)
	local halfX = self.width / 2
	local halfY = self.height / 2
	local nextY = math.floor(self.y + (self.yspeed * dt))
	local nextX = math.floor(self.x + (self.xspeed * dt))
	if(self:isColliding(map, nextX + halfX, self.y - halfY))
	    or(self:isColliding(map, nextX + halfX, self.y + halfY - 1)) -- to the right...
	    or(self:isColliding(map, nextX - halfX, self.y - halfY))
	    or(self:isColliding(map, nextX - halfX, self.y + halfY - 1))  then -- to the left...
		return true
	end
	return false
end

-- Check above or below depending on which way they're going
function Player:collideY(dt)
	local halfX = self.width / 2
	local halfY = self.height / 2
	local nextY = math.floor(self.y + (self.yspeed * dt))
	local nextX = math.floor(self.x + (self.xspeed * dt))
	self.onGround = false
	if self.yspeed < 0 then -- check upward
		if(self:isColliding(map, self.x - halfX, nextY - halfY))
		  or(self:isColliding(map, self.x + halfX - 1, nextY - halfY)) then
			return true
		end
	elseif self.yspeed > 0 then -- check downward
		if(self:isColliding(map, self.x - halfX, nextY + halfY))
		  or(self:isColliding(map, self.x + halfX - 1, nextY + halfY)) then
			self.onGround = true
			return true
		end
	end
	return false
end

function Player:isColliding(map, x, y)
    -- get tile coordinates
    local layer = map.tl["Walls"]
    local tileX, tileY = math.floor(x / map.tileWidth), math.floor(y / map.tileHeight)
    
    -- grab the tile at given point
    local tile = layer.tileData(tileX, tileY)
    
    -- return true if the point overlaps a solid tile
    return not(tile == nil)
end

function Player:action(action)
	if action == "up" then
		self:jump()
	elseif action == "left" or action == "right" then
		-- Save last direction for conflicts
		if action == "left" then
			self.lastDir = -1
		else
			self.lastDir = 1
		end
		self:setState(PC_MOVING)
	elseif action == "down" then
		-- Nothing yet
	end
end

--- Moves the player to position x,y and resets velocity in both axes
function Player:warp(x,y)
	self.x, self.y = x,y
	self.xspeed, self.yspeed = 0,0
	self:setState(PC_STANDING)
end

--- Sets a new starting location for the player
function Player:newStart(x,y)
	self.xstart, ystart = x, y
end

--- Makes the player jump
function Player:jump()
	if self.onGround == true and self.state ~= PC_DEAD then
		self.yspeed = -JUMP_POWER
	end
end

--- Sets the current state of the player
function Player:setState(state)
	self.state = state
end

function Player:draw()
	-- Floor position
	self.flx = math.floor(self.x)
	self.fly = math.floor(self.y)

	if self.state == PC_MOVING then
		-- Draw player
		if self.onGround == false then
			self.anim:draw(self.flx, self.fly, 0, 1, 1, 10, 10)
		elseif math.abs(self.xspeed) < 30 then
			self.anim:draw(self.flx, self.fly, 0, 1, 1, 10, 10)
		else
			self.anim:draw(self.flx, self.fly, 0, 1, 1, 10, 10)
		end
	elseif self.state == PC_STANDING then
		self.anim:draw(self.flx, self.fly, 0, 1, 1, 10, 10)
	-- Dead from overheating
	elseif self.state == PS_DEAD then
		-- something here eventually
	end
end