--- Checks boundaries of variable
-- @param x Variable to check
-- @param min Minimum variable can be
-- @param max Maximum variable can be
-- @return Minimum or maximum if past, otherwise x
function math.clamp(x, min, max)
	return x < min and min or (x > max and max or x)
end

--- Collides two AABBs
-- @param b1 First bounding box
-- @param b2 Second bounding box
-- @return True if they overlap
function collideBoxes(b1,b2)
	if b1.x > b2.x+b2.w or b1.x+b1.w < b2.x
	or b1.y > b2.y+b2.h or b1.y+b1.h < b2.y then
		return false
	else
		return true
	end
end

--- Returns a random entry in a table
function table.random(t)
	return t[math.random(#t)]
end

--- Returns positive or negative
function math.sign(n)
	if n < 0 then
		return -1
	elseif n > 0 then
		return 1
	else
		return 0
	end
end

--- Returns rounded number
function math.round(x)
	return math.floor(x+0.5)
end

--- Checks the x-axis for a collision
-- @param self The object you're checking for a collision
-- @return True if there is a collision, and the object collided with
function collideX(self)
	if self.xspeed == 0 then return end

	local collision = false
	
	local last -- Last object collision detected

	for _, yoff in ipairs({self.corners[3], (self.corners[3]+self.corners[4])/2, self.corners[4]}) do
		for _, xoff in ipairs({self.corners[1], self.corners[2]}) do
			-- Collide with solid tiles
			if map:collidePoint(self.x+xoff, self.y+yoff) then
				collision = true
				local cx = math.floor((self.x+xoff)/16)*16
				if self.xspeed > 0 then
					self.x = cx-self.corners[2]-0.0001
				else
					self.x = cx+16-self.corners[1]
				end
			end
			-- Collide with objects
			for i,v in ipairs(map.objects) do
				if v.solid == true and self:collideBox(v:getBBox()) then
					collision = true
					last = v
					local bbox = v:getBBox()
					if self.xspeed > 0 then
						self.x = bbox.x-self.corners[2]-0.0001
					else
						self.x = bbox.x+bbox.w-self.corners[1]+0.0001
					end
				end
			end
		end
	end
	
	return collision, last
end

--- Checks the y-axis for a collision
-- @param self The object you're checking for a collision
-- @return True if there is a collision, and the object collided with
function collideY(self)
	if self.yspeed == 0 then return end
	self.onGround = false

	local collision = false

	for _, yoff in ipairs({self.corners[3], self.corners[4]}) do
		for _, xoff in ipairs({self.corners[1], self.corners[2]}) do
			-- Collide with solid tiles
			if map:collidePoint(self.x+xoff, self.y+yoff) then
				collision = true
				local cy = math.floor((self.y+yoff)/16)*16
				if self.yspeed > 0 then
					self.y = cy-self.corners[4]-0.0001
					self.onGround = true
				else
					self.y = cy+16-self.corners[3]
				end
			end
		end
	end

	return collision
end

--- Caps a function with a minimum and a maximum
--- Essentially the same as clamp
-- @return val if smaller than max, larger than min
-- @return max if smaller than val that is larger than min
-- @return min if larger than val that is smaller than max
function cap(val, min, max)
	return math.max(math.min(val, max), min)
end

--- Just draws a box
-- @param x Horizontal location
-- @param y Vertical location
-- @param w Width of box
-- @param h Height of box
function drawBox(x,y,w,h)
	lg.setColor(30,23,18)
	lg.rectangle("fill",x+1,y+1,w-2,h-2)
	lg.setColor(255,255,255)
	-- Draw sides
	lg.drawq(img.menu_box, quad.box_left, x, y+1, 0, 1, (h-2))
	lg.drawq(img.menu_box, quad.box_left, x+w, y+1, 0, -1, (h-2))
	lg.drawq(img.menu_box, quad.box_top,  x+1, y, 0, (w-2), 1)
	lg.drawq(img.menu_box, quad.box_top,  x+1, y+h, 0, (w-2), -1)
	-- Draw corners
	lg.drawq(img.menu_box, quad.box_corner, x,y)
	lg.drawq(img.menu_box, quad.box_corner, x+w,y, 0, -1, 1)
	lg.drawq(img.menu_box, quad.box_corner, x,y+h, 0, 1, -1)
	lg.drawq(img.menu_box, quad.box_corner, x+w,y+h, 0, -1, -1)
end

--- Converts time in seconds to a clock string
-- @param time The current time in seconds
-- @return "HH:MM:SS"
function secondsToString(time)
	local hours =   string.format("%02d", math.floor(time / 60^2))
	local minutes = string.format("%02d", math.floor((time % 60^2) / 60))
	local seconds = string.format("%02d", math.floor(time % 60))

	return hours .. ":" .. minutes .. ":" .. seconds
end
